$(document).ready(function() {
	var sidebar = $(".sidebar");
	var sidebarOffset = sidebar.offset().top - 100; 

// ==============================================================
// === Set the sidebar to fixed / sticky on scroll
// ==============================================================

	$(window).scroll(function(){
		if ($(window).height() <= sidebar.height())
			return; 

		if ($(window).scrollTop() >= sidebarOffset) 
			sidebar.addClass("sticky"); 
		else 
			sidebar.removeClass("sticky");
		
	});

// ==============================================================
// === If form inputs have text content, ensure that the label 
// === is set to small so that it doesn't overlap the input 
// === text content. 
// ==============================================================

	$(".form-input .txt-input").each(function(){
		if ($(this).val() != "")
			$(this).next().addClass('label-small'); 
		else
			$(this).next().removeClass('label-small'); 
	});

// ==============================================================
// === When the text input changes, run the test again to see
// === if we need to set the label as small
// ==============================================================

	$(".form-input .txt-input").change(function(){
		if ($(this).val() != "")
			$(this).next().addClass('label-small'); 
		else
			$(this).next().removeClass('label-small'); 
	});

// ==============================================================
// === Make our custom select modal so that it will close on
// === any click outside of the select 
// ==============================================================

	$("form .form-input .txt-input").focus(function(){
		var form = $(this).parents('form').first(); 
		var idFor = $(this).attr('id'); 
		var info = form.find(".info-item[for='" + idFor + "']").first(); 
		info.css({'display' : 'block'}); 
	});

	$("form .form-input .txt-input").blur(function(){
		var form = $(this).parents('form').first(); 
		var idFor = $(this).attr('id'); 
		var info = form.find(".info-item[for='" + idFor + "']").first(); 
		info.css({'display' : 'none'}); 
	});

// ==============================================================
// === Custom select code =======================================
// === Select elements are very restrictive in regards to
// === styling, animations and structure. Therefore, in order
// === to develop our desired design into functional code, we
// === have to build our own select object. This select is
// === modal, keyboard accessible, and allows for more custom
// === structural definitions.
// ==============================================================

// ==============================================================
// === Make our custom select modal so that it will close on
// === any click outside of the select 
// ==============================================================

	$(document).click(function(e){
		var container = $(".form-select"); 

		if(!container.is($(e.target)) && container.has($(e.target)).length == 0) {
			$(".form-select div.select-item.open").each(function(){
				closeSelect($(this), false);
			});
		} else { 
			// TO DO: Fix bug where if you click on a separate form-select, any previously opened form-selects don't close / lose focus state
		}
	});

// ==============================================================
// === Ensure that we move the focus to the select input even
// === when we click on the arrow span floating on top of the
// === input element
// ==============================================================

	$(".form-select .select-item span").click(function(){
		$(this).prev().focus();
	});

// ==============================================================
// === When the input is focused in the select, we want to
// === add the focused class to the entire select container. 
// === This class only is set while the focus is on the input
// === so it doesn't correspond with when the user is 
// === interacting with the select as a whole. In order to 
// === determine if they are, you have to use a combination
// === of the focused and open classes. These functions also 
// === show / hide the form information on focus
// ==============================================================

	$(".form-select input").focus(function(){
		$(this).parents(".form-select").addClass("focused");
		$(this).next().addClass("focused"); 
		
		var form = $(this).parents('form').first(); 

		if (form.is('form')) {
			var idFor = $(this).parent().attr('id'); 
			var info = form.find(".info-item[for='" + idFor + "']").first(); 
			info.css({'display' : 'block'}); 
		}
	});

	$(".form-select input").blur(function(){
		$(this).parents(".form-select").removeClass("focused");
		$(this).next().removeClass("focused"); 

		var form = $(this).parents('form').first(); 
		
		if (!$(this).parents(".form-select").hasClass("open") && form.is("form")) {
			var idFor = $(this).parent().attr('id'); 
			var info = form.find(".info-item[for='" + idFor + "']").first(); 
			info.css({'display' : 'none'}); 
		}
	});

// TODO: When a user opens the select and moves focus to one of the buttons, and then clicks on a different input outside of the select, the info-item for the select isn't correctly hidden because in this case the blur is on the button and not the input

// ==============================================================
// === This function deals with the logic of either opening
// === or closing the select on click, and (if the click was
// === on a button in the select list) setting the select val
// === to the clicked item
// ==============================================================

	$(".form-select").click(function(e){
		var target = $(e.target); 
		var idFor = ""; 
		var selectItem; 

		if (target.is(".select-item input") || target.is(".select-item span")) {
			selectItem = target.parent();
			idFor = selectItem.attr("id"); 
			selectItem.find("input").first().focus();
		} else if (target.is("button")) {
			idFor = target.parent().attr("for"); 
			selectItem = $(this).find(".select-item[id='" + idFor + "']");

			if (target.attr("selectString") == undefined)
				selectItem.find("input").first().val(target.html()); 
			else
				selectItem.find("input").first().val(target.attr("selectString")); 

			selectItem.attr("value", target.attr("value"));
			selectItem.removeClass("default");
		}

		if (selectItem.hasClass("open")) 
			closeSelect(selectItem, true); 
		else
			openSelect(selectItem);
	});

// ==============================================================
// === This deals with the keyboarding functionality for the
// === form so that space and enter work, as well as jump to
// === an item in the list with letter press
// ==============================================================

	$(".form-select").keypress(function(e){
		if (e.key == ' ' || e.key == 'Enter') {
			var focusedItem = $(":focus"); 

			if (focusedItem.is("input")) {
				var focusedSelect = focusedItem.parent(); 

				if (focusedSelect.hasClass("open"))
					closeSelect(focusedSelect, true); 
				else 
					openSelect(focusedSelect);

				e.preventDefault();
			}
		}

		if (/[a-zA-Z]/.test(e.key)) {
			var openList = null; 
			var select = null; 

			select = $(this).children("div.select-item.open").first();

			if (select != null)
				openList = $(this).find(".select-list[for='" + select.attr('id') + "']").first();

			if (openList != null) {
				var listItems = openList.children("button"); 
				for (var i = 0; i < listItems.length; i++) {
					if ($(listItems[i]).text().charAt(0).toLowerCase() == e.key.toLowerCase()) {
						$(listItems[i]).focus(); 
						break;
					}
				}
			}
		}
	});

// ==============================================================
// === This deals with the second chunk of keyboarding,
// === including escape to close the select, tab fixes so
// === that tab on an open list doesn't tab you through the 
// === select list buttons, and up and down arrow navigation
// ==============================================================

	$(".form-select").keydown(function(e){
		if (e.key == "Escape") {
			$(this).children("div.select-item.open").each(function(){
				closeSelect($(this), true); 
			});
		}

		var focused = $(":focus"); 
		var openList = null; 
		var select = null; 

		select = $(this).children("div.select-item.open").first();

		if (select != null)
			openList = $(this).find(".select-list[for='" + select.attr('id') + "']").first();

		if (e.key == "Tab" && focused.is("input") && select.is("div.select-item.open")) {
			e.preventDefault(); 
			closeSelect(select, true); 
		}

		if ((e.key == "ArrowDown" || e.key == "ArrowUp") && openList != null) {
			e.preventDefault();
			
			var focusedBtn = openList.children("button:focus"); 
			if (focusedBtn.is("button") && e.key == "ArrowDown") {
				if (focusedBtn.next().is("button"))
					focusedBtn.next().focus(); 
			} else if (focusedBtn.is("button") && e.key == "ArrowUp") {
				if (focusedBtn.prev().is("button"))
					focusedBtn.prev().focus();
			} else if (e.key == "ArrowDown") 
				openList.children().first().focus(); 
		}
	}); 

// ==============================================================
// === This moves the focus to a select list button when you
// === mouseover it to prevent conflicting focus and hover 
// === states
// ==============================================================

	$(".form-select button").mouseover(function(e){
		var idFor = $(this).parent().attr("for"); 
		var select = $(this).parent().siblings("div.select-item[id='" + idFor + "']").first(); 
		
		if (select.hasClass("open"))
			$(this).focus();
	})

// ==============================================================
// === The function to open a select and focus the correct 
// === select list button based on the input value
// ==============================================================

	function openSelect(select) {
		var formParent = select.parent(); 
		var selectSiblings = formParent.children(".select-item");

		selectSiblings.each(function(){
			if ($(this).hasClass("open"))
				closeSelect($(this), true); 
		}); 

		select.addClass("open");
		formParent.addClass("open");
		
		var list = formParent.find(".select-list[for='" + select.attr('id') + "']").first(); 
		list.slideDown(); 

		if (!(select.hasClass('default'))) {
			var value = select.attr('value'); 
			list.find("button").each(function(){
				if ($(this).attr('value') == value) {
					$(this).focus();
				}
			});
		} else
			select.children("input").first().focus();
	}

// ==============================================================
// === Function to close the select. The boolean focusSelect
// === determines whether or not to bring the focus back to 
// === the select input, which is required when interactions
// === within the select cause it to close, but is not 
// === required / meant to happen when clicking outside of
// === the select object causes it to close. 
// ==============================================================

	function closeSelect(select, focusSelect) {
		select.removeClass("open");
		select.parent().removeClass("open");  

		var list = select.parent().find(".select-list[for='" + select.attr('id') + "']").first(); 

		list.slideUp(); 

		if (focusSelect)
			select.find("input").first().focus();
	}


}); 
